
#ifndef ROBOTS_H
#define ROBOTS_H

// maximum number of robots
#define ROBOTS_MAX 64

// robot types
#define ROBOT_TYPE_NONE      0
#define ROBOT_TYPE_MICROWAVE 1
#define ROBOT_TYPE_TOASTER   2

// robot states
#define ROBOT_STATE_IDLE        0
#define ROBOT_STATE_GOTO_TARGET 1
#define ROBOT_STATE_GOTO_BASE   2
#define ROBOT_STATE_BASE_1      3
#define ROBOT_STATE_BASE_2      4
#define ROBOT_STATE_HARVEST     5
#define ROBOT_STATE_ATTACK      6

// the default gas for the robots, in ms
#define ROBOT_DEFAULT_GAS 100000.0f

// the time it takes to harvest something, in ms
#define ROBOT_DEFAULT_HARVEST_TIME 2000.0f

// robot statuses
#define ROBOT_STATUS_EXPLODE   (1 << 0)
#define ROBOT_STATUS_INVISIBLE (1 << 1)

// robots
struct robot {
	int type;
	float rX, rY, rZ;
	float targetX, targetY;
	float interestingX, interestingY;
	float guardX, guardY;
	float harvestX, harvestY;
	float externalX, externalY;
	float turnTimer;
	float angle;
	float angleSpeed;
	float angleStart;
	float rXSpeed, rYSpeed, rZSpeed;
	float rXStart, rYStart, rZStart;
	int owner;
	int iD;
	int state;
	int status;
	int payload;
	int knowsAnInterestingSpot;
	int health;
	int targetBase;
	int targetRobot;
	int updatePacketNumber;
	float updatePacketTime;
	float harvestTimer;
	float gas;    // the gas is in ms
};

// all robots
extern struct robot robots[ROBOTS_MAX];

// the currently selected robot
extern int robotSelected;

void robotsInit(void);
void robotsIteration(float deltaTime);
void robotsDraw(void);
int  robotsAdd(int type, float rX, float rY, float angle, int owner, int iD, int state);
void robotsUpdate(int senderID, int robotID, int type, float rX, float rY, float rZ, float angle, int packetNumber);
void robotsRemoveMissing(int senderID, int packetNumber);
void robotsSelectNext(void);
void robotAddParticlesDust(struct robot *r, float dX, float dY);
int  robotToRobotCollision(int robot, float nX, float nY);
int  robotToTileCollision(float nX, float nY);
int  robotToBaubleCollision(float nX, float nY);

#endif
